#
# Time Drive - based on duplicity - Encrypted bandwidth efficient backup.
#
from PyQt4 import QtCore
import os
import timedrive.utils as utils
import timedrive.backupsettings as backupsettings

from timedrive.background import backgroundtask
from timedrive.backupsettings import globals 
from timedrive.utils import duplicity_interface
from timedrive.utils import log
from timedrive.utils import archiveurl

class BackupJob(backgroundtask.BackgroundTask):
	"""
	Class that executes backup jobs and updates user on progress.  Designed to run
	as a baackground thread so that the main GUI instance remains responsive.
	"""
	
	def __init__(self, parent = None):
		"""
		Initialization function for BackupJob
		
		@type parent: BackgroundTask Object
		@param parent:
		"""
		super(BackupJob, self).__init__(parent)
		self.exiting = False
	
	def StartBackup(self, settings):
		"""
		Start a backup job, either from the GUI or from the command line.
		
		@type settings: Settings Object
		@param settings:
		"""
		self.settings = settings
		self.start()

	def isValidExclude(self, exclude):
		"""
		check if exclude type is know and validate the exclude
		
		@type exclude: ExcludeItem Object
		@param exclude: the exclude of type ExcludeItem
		
		@rtype: Boolean
		@return result
		"""
		if exclude.Type == exclude.FOLDER:
			if os.path.exists(exclude.Item):
				return True
			else:
				log.Error(str(exclude.Item) + " could not be found, ignoring")

		elif exclude.Type == exclude.FILE:
			if os.path.isfile(exclude.Item):
				return True
			else:
				log.Error(str(exclude.Item) + " could not be found, ignoring")

		elif exclude.Type == exclude.REGEXP:
			if utils.isValidRegExp(exclude.Item):
				return True
			else:
				log.Error(str(exclude.Item) + " invalid REGEXP, ignoring")
		else:
			raise Exception(_("Invalid exclude type"))
	
		return False
			
	def process_duplicityError(self, task, result_err, folder):
		"""
		Error processing of duplicity
		
		@type task: String
		@param task: the name of task where the error occured
		@type result_err: String
		@param result_err: message that contains the error
		@type folder: IncludeItem Object
		@param folder: the includeitem for whitch the error occured
		"""
		if self.settings.Mode == globals.GuiMode:
			handle = utils.handle_error(result_err)
			if handle == 1:
				self.questionPrompt(str(task) + " " +str(folder.FolderName), _("Computer name changed"))
				duplicity_interface.allow_source_mismatch = True
			elif handle == 99:
				self.taskError(task, _("Unknow Fatal error ")+str(folder.FolderName))
			elif handle == 0:
				self.taskError(task, _("Backend Error occurd with folder ")+str(folder.FolderName))
			else:
				self.taskError(task, _("Unknow Error occured ")+str(folder.FolderName))
				log.Error(result_err)
		elif self.settings.Mode == globals.BatchMode:
			self.settings.ErrorOccurred = True
			backupsettings.writeSettings(self.settings)
			log.Error(result_err)	

	def doBackupJob(self, folder, archiveUrl, gnuPassphrase):
		"""
		Does the backup job for a single folder, archive url already build
		
		@type folder: IncludeItem Object
		@param folder:
		@type archiveUrl: String
		@param archiveUrl:
		@type gnuPassphrase: String
		@param gnuPassphrase:
		"""
		#reset fatal error options
		duplicity_interface.allow_source_mismatch = False
	
		combinedExcludeList = []

		folderName = os.path.basename(str(folder.FolderName))
		
		if self.settings.Options_ExcludeHidden == True:
			combinedExcludeList.append(globals.EXCLUDEHIDDEN)
		
		if self.settings.Options_FullBackupFrequencyEnabled == True:
			FullBackupInterval = self.settings.Options_FullBackupFrequency
		else:
			FullBackupInterval = None
		
		for e in self.settings.ExcludeList:
			if self.isValidExclude(e):
				combinedExcludeList.append((e.Item, e.Type))

		for e in folder.ExcludeList:
			if self.isValidExclude(e):
				combinedExcludeList.append((e.Item, e.Type))
		
		try:
			# Start the Backup Job
			if self.settings.Options_EncryptFilesEnabled == False:
				gnuPassphrase = None
			
			if self.settings.AmazonS3_Location == "European":
				S3_BucketType = self.settings.AmazonS3_Location
			else:
				S3_BucketType = None
			
			self._restart = True

			while self._restart:
				self._restart = False
				if self.settings.AmazonS3Backup == True:
					result_out, result_err = duplicity_interface.start_backup(str(folder.FolderName), 
						archiveUrl, gnuPassphrase, combinedExcludeList, FullBackupInterval,
						self.settings.AmazonS3_AccessId, self.settings.AmazonS3_SecretKey,
						S3_BucketType)
				else:
					result_out, result_err = duplicity_interface.start_backup(str(folder.FolderName), 
						archiveUrl, gnuPassphrase, combinedExcludeList, FullBackupInterval)
			
				if result_out:
					log.Info(str(folder.FolderName) + _(" Backup Successful using ") + utils.backupTypeToText(folder) + "\n" + result_out)

				if result_err and result_err != "":
					self.process_duplicityError("BackupJob", result_err, folder)
			
		except:
			message = QtCore.QString(_("Failed: ") + folder.FolderName)
			log.Error(str(message))
			self.taskError("Backup Job", "error", message)

	def run(self):
		"""
		The run it self, to process each folder in the folder list.
		"""
		numItems = len(self.settings.IncludeList)
		self.setupTaskProgress(numItems, _("Backup Running ... "))
		progress = 0	

		for folder in self.settings.IncludeList:
			progress += 1

			au = archiveurl.ArchiveUrl_FromSettings(folder, self.settings)
			if au.hostAvailable:
				self.doBackupJob(folder, str(au.archiveUrl), str(au.gnuPassphrase))
			else:
				message = QtCore.QString(_("Failed: ") + folder.FolderName)
				log.Error(_("Backup Job: host down ") + str(message))
				self.taskError("Backup Job", "host down", message)				
			
			self.updateTaskProgress(progress)
		
		self.updateTaskProgress(numItems)
		self.taskComplete("Backup Job", _("Backup Completed"))
